package com.company.sample.app;

import org.geotools.api.referencing.FactoryException;
import org.geotools.api.referencing.crs.CoordinateReferenceSystem;
import org.geotools.api.referencing.operation.MathTransform;
import org.geotools.api.referencing.operation.TransformException;
import org.geotools.geometry.jts.JTS;
import org.geotools.referencing.CRS;
import org.locationtech.jts.geom.Geometry;

public final class AppCRSs {

    public static final String EPSG_4326 =
            """
                    GEOGCS["WGS 84",
                        DATUM["WGS_1984",
                            SPHEROID["WGS 84",6378137,298.257223563,
                            AUTHORITY["EPSG","7030"]],
                        AUTHORITY["EPSG","6326"]],
                        PRIMEM["Greenwich",0,
                            AUTHORITY["EPSG","8901"]],
                        UNIT["degree",0.0174532925199433,
                            AUTHORITY["EPSG","9122"]],
                        AUTHORITY["EPSG","4326"]]
                    """;

    public static final String EPSG_21781 =
            """
                    PROJCS["CH1903 / LV03",
                        GEOGCS["CH1903",
                            DATUM["CH1903",
                                SPHEROID["Bessel 1841",6377397.155,299.1528128],
                                TOWGS84[674.374,15.056,405.346,0,0,0,0]],
                            PRIMEM["Greenwich",0,
                                AUTHORITY["EPSG","8901"]],
                            UNIT["degree",0.0174532925199433,
                                AUTHORITY["EPSG","9122"]],
                            AUTHORITY["EPSG","4149"]],
                        PROJECTION["Hotine_Oblique_Mercator_Azimuth_Center"],
                        PARAMETER["latitude_of_center",46.9524055555556],
                        PARAMETER["longitude_of_center",7.43958333333333],
                        PARAMETER["azimuth",90],
                        PARAMETER["rectified_grid_angle",90],
                        PARAMETER["scale_factor",1],
                        PARAMETER["false_easting",600000],
                        PARAMETER["false_northing",200000],
                        UNIT["metre",1,
                            AUTHORITY["EPSG","9001"]],
                        AXIS["Easting",EAST],
                        AXIS["Northing",NORTH],
                        AUTHORITY["EPSG","21781"]]
                    """;

    public static final String EPSG_27700 =
            """
                    PROJCS["OSGB36 / British National Grid",
                        GEOGCS["OSGB36",
                            DATUM["Ordnance_Survey_of_Great_Britain_1936",
                                SPHEROID["Airy 1830",6377563.396,299.3249646]],
                            PRIMEM["Greenwich",0,
                                AUTHORITY["EPSG","8901"]],
                            UNIT["degree",0.0174532925199433,
                                AUTHORITY["EPSG","9122"]],
                            AUTHORITY["EPSG","4277"]],
                        PROJECTION["Transverse_Mercator"],
                        PARAMETER["latitude_of_origin",49],
                        PARAMETER["central_meridian",-2],
                        PARAMETER["scale_factor",0.9996012717],
                        PARAMETER["false_easting",400000],
                        PARAMETER["false_northing",-100000],
                        UNIT["metre",1,
                            AUTHORITY["EPSG","9001"]],
                        AXIS["Easting",EAST],
                        AXIS["Northing",NORTH],
                        AUTHORITY["EPSG","27700"]]
                    """;

    public static <T extends Geometry> T convert(Geometry source, String sourceSrcWkt, String targetCrcWkt) {
        try {
            CoordinateReferenceSystem sourceCRS = CRS.parseWKT(sourceSrcWkt);
            CoordinateReferenceSystem targetCRS = CRS.parseWKT(targetCrcWkt);
            MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
            return (T) JTS.transform(source, transform);
        } catch (FactoryException e) {
            throw new IllegalStateException("Cannot parse SRC from WKT value", e);
        } catch (TransformException e) {
            throw new IllegalStateException("Cannot transform", e);
        }
    }
}
