package com.company.sample.view.pointbng;

import com.company.sample.entity.PointBng;
import com.company.sample.view.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.HasValueAndElement;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.Route;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.model.*;
import io.jmix.flowui.view.*;

@Route(value = "pointBngs", layout = MainView.class)
@ViewController("PointBng.list")
@ViewDescriptor("point-bng-list-view.xml")
@LookupComponent("pointBngsDataGrid")
@DialogMode(width = "64em")
public class PointBngListView extends StandardListView<PointBng> {

    @ViewComponent
    private DataContext dataContext;

    @ViewComponent
    private CollectionContainer<PointBng> pointBngsDc;

    @ViewComponent
    private InstanceContainer<PointBng> pointBngDc;

    @ViewComponent
    private InstanceLoader<PointBng> pointBngDl;

    @ViewComponent
    private VerticalLayout listLayout;

    @ViewComponent
    private FormLayout form;

    @ViewComponent
    private HorizontalLayout detailActions;

    @Subscribe
    public void onInit(final InitEvent event) {
        updateControls(false);
    }

    @Subscribe("pointBngsDataGrid.create")
    public void onPointBngsDataGridCreate(final ActionPerformedEvent event) {
        dataContext.clear();
        PointBng entity = dataContext.create(PointBng.class);
        pointBngDc.setItem(entity);
        updateControls(true);
    }

    @Subscribe("pointBngsDataGrid.edit")
    public void onPointBngsDataGridEdit(final ActionPerformedEvent event) {
        updateControls(true);
    }

    @Subscribe("saveBtn")
    public void onSaveButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.save();
        pointBngsDc.replaceItem(pointBngDc.getItem());
        updateControls(false);
    }

    @Subscribe("cancelBtn")
    public void onCancelButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.clear();
        pointBngDl.load();
        updateControls(false);
    }

    @Subscribe(id = "pointBngsDc", target = Target.DATA_CONTAINER)
    public void onPointBngsDcItemChange(final InstanceContainer.ItemChangeEvent<PointBng> event) {
        PointBng entity = event.getItem();
        dataContext.clear();
        if (entity != null) {
            pointBngDl.setEntityId(entity.getId());
            pointBngDl.load();
        } else {
            pointBngDl.setEntityId(null);
            pointBngDc.setItem(null);
        }
    }

    private void updateControls(boolean editing) {
        form.getChildren().forEach(component -> {
            if (component instanceof HasValueAndElement<?, ?> field) {
                field.setReadOnly(!editing);
            }
        });

        detailActions.setVisible(editing);
        listLayout.setEnabled(!editing);
    }
}